function d0_3D = CapacityDimension3D(filename, ell)

% This Matlab function is supplementary document of manuscript in 
% Soft Matter
% Manuscript title: Reconstructing the fractal dimension of granular
% aggregates from light intensity spectra
% Manuscrip ID: SM-ART-07-2015-001885
% Authors: Tang F.H.M. and F. Maggi
% Date: 2015 September
%
% CapacityDimension3D function returns the 3D capacity dimension of an aggregate from
% its 2D projection.
% Prior to using this function, grayscale image has to be converted to
% binary, as this function only allows input of binary image.
%
% d0_3D = CapacityDimension3D(FILENAME, ELL) reads a binary image from the
% file specified by FILENAME (e.g.,'BW_CCA_1-7323.bmp'), resize the
% image to the desired resolution/dimensionless aggregate size ELL (e.g.,
% ELL=1024) and returned the estimated capacity dimension.
% Note that ELL has to be greater than orginal image size.
% Two grayscale and two binary images of numerical aggregates are
% provided as examples
% CCA_1-7323 is cluster-cluster aggregate with exact d0_3D = 1.7323
% SCA_2-7086 is self-correlated aggregate with exact d0_3D = 2.7086

image01 = imread(filename); %raw black and white image
%%% Check if the image is black and white
if max(max(image01))> 1;
    disp('Error-- Please input a binary image of aggregate.')
    return
end

image02 = imfill(image01, 'holes'); %fill holes

%%% Check the size of the raw image
if mean(size(image01,1),size(image01,2))> ell
    sprintf('Error-- Desired resolution ell has to be greater than orginal image size = %2.2d.',mean(size(image01,1),size(image01,2)))
    return
end

%%% Resize image to desire resolution ell
image03= imresize(image02, [ell ell],'nearest');
image03= uint8(image03);

%%% Dimensionless area of aggregate N_A
[s1 t1] = find(image03==1);
N_A = length(s1);

%%% Dimensionless perimeter of aggregate N_P
image04= bwperim(image03);
[s2 t2] = find(image04==1);
N_P = length(s2);

%%% Outermost perimeter-based fractal dimension dP_0
dP_0 = 2* log10(N_P)/log10(N_A); %  Eq.(4)

%%% Perimeter-based fractal dimension at I=1
Z= 2* log10(4*ell - 4)/log10(ell^2); % Eq.(5)

%%% Alpha with \tilde{alpha}=4
alpha= (4-2)*dP_0/(Z-2) + 2*(Z-4)/(Z-2); %Eq.(14)

%%%% Analytical dP_I spectrum, dP_I = AI^2+BI+C (Eq. (6))
A = -1* alpha* (Z - dP_0)/(2-alpha); %Eq. (12)
B = Z - dP_0 - A; %Eq. (8)
C =  dP_0; %Eq. (7)

%%% Check if the spetrum exceed the global maximum =2;
I_max= -1*B/(2*A);
dP_I_max= A * I_max^2 + B .* I_max + C;

if dP_I_max > 2 % dP_I_max = 2
    tmpA = Z - dP_0;
    tmpB = 2*(dP_0 - 2);
    tmpC = 2 - dP_0;
    
    tmpI(1) = (-tmpB + sqrt(tmpB^2- 4*tmpA*tmpC))/2*tmpA;
    tmpI(2) = (-tmpB - sqrt(tmpB^2- 4*tmpA*tmpC))/2*tmpA;
   
    tmpI_max = tmpI(tmpI>0);
    
    if length(tmpI_max)>1
        tmpI_max = tmpI(tmpI<1);
        tmpI_max = min(tmpI_max);
    end
    
    A= (dp_0 - Z)/(2*tmpI_max -1); %Eq. (16)
    B= Z - dP_0 - A;
end

%%% Function f(ell)= beta1*dp_I^2 + beta2*dp_I + beta3 Eq. (19)
ell_given   = [64 128 256 512 1024 2048 4096 8192];
beta1_given = [3.1239    4.1186    5.0458    6.1108    7.5930  10.4402   13.7207   16.4458];
beta2_given = [-9.7720  -12.4070  -14.8611  -17.6664  -21.6212  -29.3393  -38.0654  -44.7104];
beta3_given = [8.3318   10.0342   11.5941   13.3657   15.9176   21.0255   26.6887   30.6337];

%%%% Linear interpolation to find beta1, beta2 and beta3 Fig.(4b)
beta1= interp1(ell_given, beta1_given, ell); 
beta2= interp1(ell_given, beta2_given, ell);
beta3= interp1(ell_given, beta3_given, ell);

%%% Determine optimum perimeter-based fractal dimension dP_I_opt 
%% Equation 1: dP_I = AI^2+BI+C
%% Equation 2: I = beta1*dp_I^2 + beta2*dp_I + beta3
%% Solve for intersect between Equation 1 and Equation 2 Eq.(17)
syms tmpdP_I_opt
tmpdP_I_opt = solve(A*(beta1*tmpdP_I_opt^2+beta2*tmpdP_I_opt+beta3)^2 + B*(beta1*tmpdP_I_opt^2+beta2*tmpdP_I_opt+beta3)+C-tmpdP_I_opt);
tmpdP_I_opt= double(tmpdP_I_opt);
%%% Select solution for dP_I_opt
tmpdP_I_opt= tmpdP_I_opt(tmpdP_I_opt==real(tmpdP_I_opt)); %% only get real numbers
tmpdP_I_opt = tmpdP_I_opt(tmpdP_I_opt>=Z); %%% dP has to be greater than Z
tmpdP_I_opt = tmpdP_I_opt(tmpdP_I_opt<=2); %%% dP has to be smaller than 2

if isempty(tmpdP_I_opt)==1  %%% no real solution (i.e., dP_I_opt>dP_I_max), then I_opt = I_max
    %% Find I_max
    I_max= -1* B/ (2 * A);
    dP_I_opt= A * I_max^2 + B * I_max + C;

else
    %%% Check if I_opt is in the range of 0 and 1
    I_opt= beta1*tmpdP_I_opt.^2+beta2*tmpdP_I_opt+beta3;
    [ss tt] = find(I_opt>=0 & I_opt<=1);
    
    tmpdP_I_opt = tmpdP_I_opt(tt);
    
    %%% Get the maximum if there are more than one solution
    dP_I_opt= max(tmpdP_I_opt); 

end

%%% 3D Capacity Dimension d0_3D Eq.(2)
k= Z * (Z - 1) + 1;

a = 9 * (Z - ((2* k ^2 - 9 * Z)/(k^2 - 9)));
b = (2* k^2 - 9*Z)/(k^2 - 9);

d0_3D= sqrt(a ./ (dP_I_opt - b));

